#include <init.h>

/* Gnre un point alatoire sur le damier.
 * Si le boolen ferme vaut VRAI, le gnrateur alatoire est ferm et aucun
 * point alatoire n'est ralis. Fonctionnalit utilise quand le programme
 * se termine.
 * /dev/urandom n'est ainsi ouvert qu'une seule fois.
 */
bool random_point(Point * p, bool ferme, Damier * dam)
{
    static FILE * random = NULL;

    if ( random == NULL ) {
	/* Premire utilisation, on ouvre /dev/urandom */
	if ( (random = fopen("/dev/urandom","r")) == NULL ) {
	    fprintf(stderr,"Impossible d'ouvrir /dev/urandom.\n");
	    return FAUX;
	}
    }

    if ( ferme ) {
	fclose(random);
	return VRAI;
    }

    /* Lecture de 2 valeurs alatoires */
    if ( fread(&(p->x),sizeof(int),1,random) != 1 ) {
	fprintf(stderr,"Impossible de lire dans /dev/urandom.\n");
	p->x = 0;
	return FAUX;
    }

    if ( fread(&(p->y),sizeof(int),1,random) != 1 ) {
	fprintf(stderr,"Impossible de lire dans /dev/urandom.\n");
	p->y = 0;
	return FAUX;
    }

    /* Ajustement des valeurs alatoires */
    p->x %= dam->colonnes;
    p->y %= dam->lignes;

    /* On prend l'oppos d'une valeur si elle est ngative */
    if ( p->x < 0 )
	p->x = - p->x;
    if ( p->y < 0 )
	p->y = - p->y;

    return VRAI;
}



/* Initialise Curses :
 * Passe le terminal en mode brut et en mode non-bloquant
 * Active les touches KEY_*
 * Empche l'affichage sur le terminal quand on appuie sur une touche
 * Active les couleurs dans ncurses
 * Dsactive l'affichage du curseur
 */
void init_curses(void)
{
    /* Initialise curses */
    initscr();
    /* Passage en mode brut puis non-bloquant :
     * - brut : le terminal recoit directement l'appui sur les touches, sans
     * passer par un cache.
     * - non-bloquant : un appel de getch() n'attend pas que l'utilisateur
     * appuie sur une touche.
     */
    cbreak();
    nodelay(stdscr, TRUE);
    /* Pour avoir les touches flches */
    keypad(stdscr, TRUE);
    /* Pour viter que quelque chose s'affiche pendant l'appui d'une touche */
    noecho();
    /* Pour les couleurs */
    start_color();
    /* Pour ne pas afficher le curseur dans le terminal */
    curs_set(0);

    /* Couleurs */
    init_pair(BLANC, COLOR_WHITE, COLOR_WHITE);
    init_pair(ROUGE, COLOR_RED, COLOR_RED);
    init_pair(MAGENTA, COLOR_MAGENTA, COLOR_MAGENTA);
    init_pair(VERT, COLOR_GREEN, COLOR_GREEN);
}

/* Cr une fentre nomme nibbler pour mlvlib */
bool init_mlv(void ** mlvstr)
{
    if ( (*mlvstr = malloc(sizeof(MlvType))) == NULL ) {
	fprintf(stderr,"ERREUR : Impossible d'initialiser la MlvLib");
	return FAUX;
    }

    create_window(":0", MLV_LARG, MLV_HAUT, *mlvstr);
    name_window("Nibbler", *mlvstr);

    return VRAI;
}

/* Initialisation du damier */
bool init_damier(Damier * dam)
{
    int i,j;

    /* Allocation des lignes */
    if ( (dam->tab = malloc(sizeof(Case *) * dam->lignes)) == NULL )
	return FAUX;
    for ( i=0 ; i < dam->lignes ; i++ ) {
	/* Allocation des colonnes de la ligne i */
	if ( (dam->tab[i] = malloc(sizeof(Case) * dam->colonnes)) == NULL )
	    return FAUX;
	/* Initialisation de la ligne */
	for ( j=0 ; j < dam->colonnes ; j++ )
	    dam->tab[i][j] = VIDE;
    }

    return VRAI;
}

/* Initialisation des proies */
bool init_proies(Damier * dam)
{
    int i;
    Point p = { 0, 0 };

    for ( i=0 ; i < dam->nbproies ; i++ ) {
	/* On cherche une case innocupe alatoirement */
	do { random_point(&p,FAUX,dam);
	} while ( dam->tab[p.y][p.x] != VIDE );
	dam->tab[p.y][p.x] = PROIE;
    }
    
    return VRAI;
}

/* Initialisation du ver */
bool init_ver(Ver * ver, Damier *dam)
{
    if ( (ver->pos = malloc(sizeof(Point) * dam->nbproies)) == NULL )
	return FAUX;

    /* On cherche une case alatoire innocupe pour le dpart */
    do { random_point(&(ver->pos[0]),FAUX,dam);
    } while ( dam->tab[ver->pos[0].y][ver->pos[0].x] != VIDE );

    ver->sens = Pause;
    ver->tete = ver->queue = 0;
    ver->taille_ver = 1;
    ver->taille_file = dam->nbproies;

    return VRAI;
}

/* "Parse" les options de la ligne de commande */
void options(int argc, char **argv, Damier * dam, Ver * ver, Affichage * aff)
{
    char c;
    int val;

    while ( (c = getopt(argc, argv, "a:h:l:p:")) != -1) {
	switch(c) {
	    /* Affichage */
	    case 'a':
		if ( strcmp(optarg,"MLV") == 0 )
		    *aff = MLV;
		else if ( strcmp(optarg,"TXT") == 0 )
		    *aff = TXT;
		else
		    printf("%s n'est pas un mode d'affichage\n",optarg);
		break;
	    /* Hauteur */
	    case 'h':
		val = atoi(optarg);
		if ( val < 5 )
		    printf("%s n'est pas une hauteur de damier valide\n",
			    optarg);
		else
		    dam->lignes = val;
		break;
	    /* Largeur */
	    case 'l':
		val = atoi(optarg);
		if ( val < 5 )
		    printf("%s n'est pas une largeur de damier valide\n",
			    optarg);
		else
		    dam->colonnes = val;
		break;
	    /* Nombre de proies */
	    case 'p':
		val = atoi(optarg);
		if ( val < 5 || val > (dam->colonnes * dam->lignes - 1) )
		    printf("%s n'est pas un nombre de proies valide\n",optarg);
		else
		    dam->nbproies = val;
		break;
	    /* Aide */
	}
    }
}

/* Met les valeurs par dfaut pour le programme.
 * Parse les options passes en argument au programme.
 * Initialise le damier, les proies et le ver.
 * str_aff est un pointeur vers une structure de donnes pour l'interface
 * utilisateur.
 */
bool init(int argc, char **argv, Affichage *aff, Damier * dam, Ver * ver,
	void ** str_aff)
{
    /* Valeurs par défaut */
    *aff = TXT;
    dam->lignes = 20;
    dam->colonnes = 20;
    dam->nbproies = 30;

    options(argc, argv, dam, ver, aff);

    if ( init_damier(dam) == FAUX )
	return FAUX;

    if ( init_proies(dam) == FAUX )
	return FAUX;

    if ( init_ver(ver,dam) == FAUX )
	return FAUX;

    /* Initialisation de curses, toujours ncessaire pour grer les mouvements*/
    init_curses();

    /* Initialisation de l'interface utilisateur */
    switch(*aff) {
	case TXT:
	    /* Rien  faire, curses est obligatoirement initialis */
	    break;
	case MLV:
	    if ( init_mlv(str_aff) == FAUX )
		return FAUX;
    }
    
    return VRAI;
}

/* Libration de la file du ver */
void libere_ver(Ver * ver)
{
    free(ver->pos);
}

/* Libration du damier */
void libere_dam(Damier * dam)
{
    int i;
    for ( i=0 ; i < dam->lignes ; i++ )
	free(dam->tab[i]);
    free(dam->tab);
}

/* Libre toutes les donnes alloues pendant l'utilisation du programme */
void libere(Damier * dam, Ver * ver, Affichage aff, void ** str_aff)
{
    /* Fin de curses */
    endwin();

    /* Terminaison de l'affichage */
    switch(aff) {
	case TXT:
	    break;
	case MLV:
	    close_window(*str_aff);
	    free(*str_aff);
	    *str_aff = NULL;
	    break;
    }

    libere_ver(ver);
    libere_dam(dam);

    /* Terminaison du gnrateur alatoire */
    random_point(NULL,VRAI,NULL);
}

