#include <moteur.h>

/* Retourne un Point contenant la prochaine position de la tte du ver
 * en fonction de sa direction actuelle
 */
Point prochaine_pos(Ver * ver)
{
    Point p = { ver->pos[ver->tete].x, ver->pos[ver->tete].y };
    
    switch(ver->sens)
    {
	case Nord: p.y--;
	    break;
	case Sud: p.y++;
	    break;
	case Ouest: p.x--;
	    break;
	case Est: p.x++;
	    break;
	case Pause:
	    break;
    }
    
    return p;
}

/* Modifie la direction du ver si une touche flche est presse. 
 * Si la touche presse est q, la fonction retourne FAUX pour permettre de 
 * quitter le jeu.
 */
bool direction(Ver * ver)
{
    int touche;

    switch ( (touche = getch()) ) {
	case KEY_LEFT: /* Gauche */
	    ver->sens = Ouest;
	    break;
	case KEY_RIGHT: /* Droite */
	    ver->sens = Est;
	    break;
	case KEY_DOWN: /* Bas */
	    ver->sens = Sud;
	    break;
	case KEY_UP: /* Haut */
	    ver->sens = Nord;
	    break;
	case 'p':
	    ver->sens = Pause;
	default:
	    break;
    }

    return ( touche == 'q' ) ? FAUX : VRAI ;
}

/* Met  jour le ver en dplacant la queue ( le ver avance )
 * ppos contenir la prochaine position du ver
 */
void deplace(Ver * ver, Point * ppos)
{
    ver->tete = ( ver->tete == ver->taille_file-1 ) ? 0 : ver->tete + 1;
    ver->pos[ver->tete] = *ppos;

    ver->queue = ( ver->queue == ver->taille_file-1 ) ? 0 : ver->queue + 1;
}

/* Met  jour le ver sans dplacer la queue ( effet d'allongement du ver )
 * ppos doit contenir la prochaine position du ver
 */
void deplace_mange(Ver * ver, Point * ppos)
{
    ver->tete = ( ver->tete == ver->taille_file-1 ) ? 0 : ver->tete + 1;
    ver->pos[ver->tete] = *ppos;
}

/* Retourne vrai ou faux selon que le ver va manger ou non une proie
 * et fait disparaitre la proie du damier s'il l'a mange
 * *p contiendra la prochaine position du ver
 */
bool mange(Ver * ver, Damier * dam, Point * p)
{
    *p = prochaine_pos(ver);

    if ( p->x >= 0 && p->y >= 0
	    && p->x < dam->colonnes && p->y < dam->lignes ) {
	if ( dam->tab[p->y][p->x] == PROIE ) {
	    dam->nbproies--;
	    ver->taille_ver++;
	    return VRAI;
	}
    }
    
    return FAUX;
}

/* Dtecte si le jeu est termin, c'est  dire si le ver "se mord", s'il 
 * dpasse du plateau de jeu ou s'il n'y a plus de proies.
 */
bool findejeu(Ver * ver, Damier * dam)
{
    int t;

    /* Plus de proies ? */
    if ( dam->nbproies == 0 )
	return VRAI;

    /* Vrifie si le serpent dborde du plateau */
    if ( ver->pos[ver->tete].x < 0
	    || ver->pos[ver->tete].x >= dam->colonnes )
	return VRAI;
    if ( ver->pos[ver->tete].y < 0
	    || ver->pos[ver->tete].y >= dam->lignes )
	return VRAI;

    /* Vrifie si le serpent se mort la queue ou non */
    if ( ver->taille_ver > 1 ) {
	t = ( ver->tete == 0 ) ? ver->taille_file - 1 : ver->tete - 1;
	do {
	    if ( t != ver->tete )
		if ( ver->pos[t].x == ver->pos[ver->tete].x
			&& ver->pos[t].y == ver->pos[ver->tete].y )
		    return VRAI;
	    t = ( t == 0 ) ? ver->taille_file - 1 : t - 1;
	} while ( t != ver->queue );
    }

    return FAUX;
}

/* Met  jour le plateau de jeu avec les nouvelles positions du ver */
void maj(Ver * ver, Damier * dam)
{
    int i,j,t;
    
    /* Vide le plateau de toutes "marques" autres que celle des proies */
    for ( i=0 ; i < dam->lignes ; i++ )
	for ( j=0 ; j < dam->colonnes ; j++ )
	    if ( dam->tab[i][j] != PROIE )
		dam->tab[i][j] = VIDE;
    

    /* Met les "marques" actuelles du ver */
    dam->tab[ver->pos[ver->tete].y][ver->pos[ver->tete].x] = TETE;
    t = ver->tete;
    for ( i=0 ; i < ver->taille_ver-1 ; i++ ) {
	t = ( t == 0 ) ? ver->taille_file - 1 : t - 1 ;
	dam->tab[ver->pos[t].y][ver->pos[t].x] = ANNEAU;	
    }
}

/* Boucle d'xcution du programme 
 * affiche, dplace le ver, mange les proies, etc 
 */
void nibbler(Damier * dam, Ver * ver, Affichage aff, void * straff)
{
    bool jeu = VRAI;
    Point p;

    while ( jeu ) {
	/* Mise  jour et affichage */
	maj(ver,dam);
	affichage(dam,aff,straff);

	/* Jeu */
	jeu = direction(ver);
	if ( ver->sens != Pause ) {
	    if ( mange(ver,dam,&p) )
		deplace_mange(ver,&p);
	    else
		deplace(ver,&p);
	    jeu = !findejeu(ver,dam) && jeu;
	}

	/* Ralentissement du jeu, pour tre jouable */
	usleep(120000);
    }

    gameover(aff,straff);
}

